/**
 * Confront section of the Farmer House scene.
 * @param {BreedingFarmController} controller The Farmer House controller.
 */
function FarmerHouseConfront(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var threatenId = arguments.callee.name + "ButtonOne";
    var intimidateId = arguments.callee.name + "ButtonTwo";
    var wrestleId = arguments.callee.name + "ButtonThree";

    var toThreaten = new FarmerHouseThreaten(ctrl);
    var toIntimidate = new FarmerHouseIntimidate(ctrl);
    var toWrestle = new FarmerHouseWrestle(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            Farmer house confront.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildThreatenChoice() {
        var disabled = ctrl.confront.has() ? "disabled" : "";
        var isSelected = ctrl.confront.get() === FarmerHouseConfrontChoice.THREATEN;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${threatenId}" ${disabled}>Threaten her</button>`;

        var tooltip = global.parser.parse(`Threaten the woman's life to keep her in check.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 180px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildIntimidateChoice() {
        var disabled = ctrl.confront.has() ? "disabled" : "";
        var isSelected = ctrl.confront.get() === FarmerHouseConfrontChoice.INTIMIDATE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${intimidateId}" ${disabled}>Intimidate her</button>`;

        var tooltip = global.parser.parse(`Shout loudly to inspire fear.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 130px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildWrestleChoice() {
        var disabled = ctrl.confront.has() ? "disabled" : "";
        var isSelected = ctrl.confront.get() === FarmerHouseConfrontChoice.WRESTLE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${wrestleId}" ${disabled}>Wrestle</button>`;

        var tooltip = global.parser.parse(`Tackle the human and wrestle her to the ground.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 180px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You loot: </div>`;
        choice += buildThreatenChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildIntimidateChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildWrestleChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + threatenId), $("#" + intimidateId), $("#" + wrestleId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerThreatenListener() {
        $("#" + threatenId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.confront.set(FarmerHouseConfrontChoice.THREATEN);
            loadNextPart(toThreaten);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerIntimidateListener() {
        $("#" + intimidateId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.confront.set(FarmerHouseConfrontChoice.INTIMIDATE);
            loadNextPart(toIntimidate);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerWrestleListener() {
        $("#" + wrestleId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.confront.set(FarmerHouseConfrontChoice.WRESTLE);
            loadNextPart(toWrestle);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.arrival.get() !== FarmerHouseArrivalChoice.CONFRONT) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        part += toThreaten.build();
        part += toIntimidate.build();
        part += toWrestle.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.arrival.get() !== FarmerHouseArrivalChoice.CONFRONT) {
            return;
        }

        toThreaten.registerListeners();
        toIntimidate.registerListeners();
        toWrestle.registerListeners();

        if (ctrl.confront.has()) {
            return;
        }

        registerThreatenListener();
        registerIntimidateListener();
        registerWrestleListener();
    };
}

var FarmerHouseConfrontChoice = {
    THREATEN: 1,
    INTIMIDATE: 2,
    WRESTLE: 3
};

